package pgdp.streams;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

public class StreamIntro {
    public static final Person p1 = new Person("Paul", 20);
    public static final Person p2 = new Person("Mia", 23);
    public static final Person p3 = new Person("Leo", 23);
    public static final Person p4 = new Person("Elias", 23);
    public static final Person p5 = new Person("Laura", 22);

    public static double calculate(List<Double> input) {
        return input.stream()
        		// negative Zahlen ausfiltern
        		.filter(i -> i < 0)
        		// quadrieren
        		.map(i -> i*i)
        		// aufsummieren
        		.reduce(0.0, (sum,currentElement) -> sum+currentElement);
    }

    public static Set<Person> toSetForEach(List<Person> input) {
        Set<Person> result = new HashSet<>();
        input.stream()
        	// jedes Element mit der Add Funktion von Set zum Set result hinzufügen
        	.forEach(result::add);
        
        // Statt Methodenreferent mit Lamda Expression
        // input.stream().forEach(person -> result.add(person);
        
        return result;
    }

    public static Set<Person> toSet(List<Person> input) {
        return input.stream()
        		// mit Collector in Set zusammenfassen
        		.collect(Collectors.toSet());
    }

    public static double average(int[] input) {
        return Arrays.stream(input)
        		// Durchschnitt berechnen
        		.average()
        		// in Double umwandeln
        		.getAsDouble();
    }

    public static double averageAge(List<Person> input) {
        return input.stream()
        		// zu Stream von primitiven int's umwandeln
        		// (mit Lamda, alternativ Methodenreferenz: .mapToInt(person::getAge)
        		.mapToInt(person -> person.getAge())
        		// Durchschnitt berechnen
        		.average()
        		// in Double umwandeln
        		.getAsDouble();
    }

    public static Map<Integer, List<Person>> groupByAgeForEach(
            List<Person> input) {
    	
    	// man kann sich eine Map wie eine Tabelle vorstellen, bei der es unterschiedliche Spaltenüberschriften gibt,
    	// die dann jeweils mit zusammengehörigen Elementen gefüllt werden
        Map<Integer, List<Person>> result = new HashMap<Integer, List<Person>>();
        
        input.forEach(person -> {														//  ––
        																				//  |
        	// aktuelle Liste mit allen Personen eines Alters aus der Map erhalten		//  |
            List<Person> l = result.get(person.getAge());								//  |
            																			//  | Das alles ist
            //prüfen, ob in der Liste schon etwas enthalten ist							//  | eine einzige Lamda
            if (l == null) {															//  | Expression.
            	// bei nicht vorhandener Liste neue Liste  erstellen					//  |
                result.put(person.getAge(), (l = new ArrayList<>()));					//  |
            }																			//  |
            // Liste die aktuelle Person hinzufügen										//  |
            l.add(person);																//  |
        });																				//  ––
        
        return result;
    }

    public static Map<Integer, List<Person>> groupByAge(List<Person> input) {
        return input.stream()
        		// mit Collector zur Map zusammenfassen (dabei wird nach Alter gruppiert)
        		.collect(Collectors.groupingBy(Person::getAge));
    }
}
